@@ -10,7 +10,15 @@
 #'
 #' @author David L. Miller
 #' @export
-create.bins <- function(data,cutpoints){
+#' @examples
+#' \dontrun{
+#' library(Distance)
+#' data(minke)
+#'
+#' # put the minke data into bins 0-1, 1-2, 2-3 km
+#' minke_cuts <- create.bins(minke[!is.na(minke$distance),], c(0,1,2,3))
+#' }
+create.bins <- function(data, cutpoints){
 
   # don't do anything if there are NAs in the distance column
   if(any(is.na(data$distance))){
@@ -25,7 +33,7 @@ create.bins <- function(data,cutpoints){
   if(!all(in.cp.ind)){
     warning("Some distances were outside bins and have been removed.")
   }
-  data <- data[in.cp.ind,]
+  data <- data[in.cp.ind, , drop=FALSE]
 
   # pull out the distances (removing the NAs for now)
   na.ind <- is.na(data$distance)
